from flask import Blueprint, request, jsonify, make_response, json
from app.device_types.models import DeviceType
from app.device_types.schemas import deviceTypes_schema
from app import db
from app.utils import token_required, delete_record, paginate_query, paginated_query
from app.device_types.utills import handle_image_upload, serialize_device_type, paginate_query, get_filtered_device_types,get_device_type_by_id

device_types = Blueprint('device_types', __name__)
@device_types.route('/api/v1/add/device_types', methods=['POST'])
@token_required(1, 2, 3)
def add_device_types(user_details):
    try:
        device_types_dict = json.loads(request.form.get('formData'))
        image_data, logo_content_type = handle_image_upload(request.files.get(
            'image')) if request.files.get('image') else (" ", " ")
        db.session.add(DeviceType(**deviceTypes_schema.load({
            'device_type_name': device_types_dict['device_type_name'].strip(),
            'measured_power': device_types_dict['measured_power'],
            'logo': image_data,
            'logo_content_type': logo_content_type,
        })))
        db.session.commit()
        return make_response(jsonify({'message': 'Device Type added successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to add Device Type', "status": "error"}), 500)

@device_types.route('/api/v1/get/device_types', methods=['GET'])
@token_required(1, 2, 3)
def get_device_types(user_details):
    try:
        query = DeviceType.query.order_by(DeviceType.device_type_name.asc())
        paginated_data, data = paginated_query(query, [DeviceType.device_type_name], None, True)
        serialized_devices_types= [serialize_device_type(device_type) for device_type in paginated_data]
        return make_response(jsonify({'devices': serialized_devices_types, "data": data}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 500)
@device_types.route('/api/v1/get/device_type_by_id/<int:device_type_id>', methods=['GET'])
@token_required(1, 2, 3)
def get_device_type(user_details, device_type_id):
    try:
        device_type = get_device_type_by_id(device_type_id)
        if device_type:
            serialized_device_type = serialize_device_type(device_type)
            logo = {
                "logo": serialized_device_type['logo'],
                "logo_content_type": serialized_device_type['logo_content_type']
                }
            serialized_device_type.pop('logo_content_type')
            serialized_device_type.pop('logo')
            return jsonify({"serialized_device_type": serialized_device_type, "logo": logo}), 200
        return jsonify({'message': 'Device type not found', "status": "error"}), 404
    except Exception as e:
        return jsonify({'message': f'Error: {str(e)}', "status": "error"}), 500
@device_types.route('/api/v1/update/device_types/<int:device_type_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_device_type(user_details, device_type_id):
    try:
        device = get_device_type_by_id(device_type_id)
        device_dict = json.loads(request.form.get('formData'))
        image_data, logo_content_type = handle_image_upload(request.files.get('image'))
        if image_data:
            device.logo = image_data
            device.logo_content_type = logo_content_type
        for key, value in deviceTypes_schema.load({
            'device_type_name': device_dict.get('device_type_name', '').strip(),
            'measured_power': device_dict.get('measured_power')
        }).items():
            setattr(device, key, value)
        db.session.commit()
        return make_response(jsonify({'message': 'Device type updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to update device type', "status": "error"}), 500)
@device_types.route('/api/v1/delete/device_types/<int:id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_device_type(current_user, id):
    return delete_record(DeviceType, current_user, id, "DeviceType", "device_type_id")
@device_types.route('/device_name', methods=['GET'])
@token_required(1, 2, 3)
def device_name(current_user):
    try:
        search = request.args.get('search')
        device_types = get_filtered_device_types(search)
        serialized_device_types = [serialize_device_type(device_type) for device_type in device_types]
        if serialized_device_types:
            return jsonify({"device_type": serialized_device_types}), 200
        return jsonify({"error": "No device types found"}), 404
    except Exception as e:
        return make_response(jsonify({"Error": str(e)}), 500)